
/* 
 * Copyright (c) 1993-2003,2012 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*------------------  Fichier gnr automatiquement ------------------*/
/*------------------  Ne pas diter manuellement !!! ------------------*/

#ifndef viman_MSG_LIB_H
#define viman_MSG_LIB_H

#include "portLib.h"
#include "csLib.h"
#include "posterLib.h"
 
#include "vimanType.h"

#define TIME_WAIT_REPLY           /*2000*/ 0
 



/*----------- Tailles des messages et boites aux lettres  ----------*/

/* Taille du buffer utilise dans la bal pour un mesage de taille size
 * (cf servMbox pour UNIX et h2rgnLib pour VXWORKS )
 */


#define BUF_SIZE(size) \
   (size + 4 - (size & 3) + sizeof(LETTER_HDR) + 8)

/**
 ** Boite aux lettres reception requetes 
 **/
#define  VIMAN_MBOX_NAME                  genomInstanceName("viman")

#define  VIMAN_MAX_RQST_SIZE              840

#define  VIMAN_MBOX_RQST_SIZE       \
 (BUF_SIZE(VIMAN_MAX_RQST_SIZE) * SERV_NMAX_RQST_ID)

/** 
 ** Taille max repliques (serveur)
 **/
#define  VIMAN_MAX_INTERMED_REPLY_SIZE   (sizeof(int))

#define  VIMAN_MAX_REPLY_SIZE             976

/** 
 ** Taille bal clients pour reception repliques 
 **/
#define  VIMAN_CLIENT_MBOX_REPLY_SIZE   \
((BUF_SIZE(VIMAN_MAX_REPLY_SIZE) \
   + BUF_SIZE(VIMAN_MAX_INTERMED_REPLY_SIZE)) \
 * CLIENT_NMAX_RQST_ID)

/*-------------------------- Liste des requetes  -------------------------*/

#define VIMAN_INIT_RQST 0
#define VIMAN_SETTRACKINGUSERMODE_RQST 1
#define VIMAN_ACTIVATETRACKING_RQST 2
#define VIMAN_SETCAMERALEFTPOSE_RQST 3
#define VIMAN_GETCAMERALEFTPOSE_RQST 4
#define VIMAN_GETCAMERARIGHTPOSE_RQST 5
#define VIMAN_SETIMAGEFRAMECORRECTION_RQST 6
#define VIMAN_SETROBOTFRAMECORRECTION_RQST 7
#define VIMAN_STOP_RQST 8
#define VIMAN_GETOBJECTPOSE_RQST 9
#define VIMAN_SETEXPORTOBJECTNAME_RQST 10
#define VIMAN_GETFULLOBJECTPOSE_RQST 11
#define VIMAN_SETTHRESHOLDARTOOLKIT_RQST 12
#define VIMAN_ABORT_RQST 13


/*---------------- PROTOTYPES DES FONCTIONS EXTERNES ------------------*/

#ifdef __cplusplus
extern "C" {
#endif

STATUS vimanClientInit (CLIENT_ID *pClientId);

STATUS vimanClientEnd (CLIENT_ID clientId);

int vimanAbortRqstSend (CLIENT_ID clientId, 
			   int *pRqstId,
			   int *activity,
			   int replyTimeOut);

int vimanAbortReplyRcv (CLIENT_ID clientId, 
			   int rqstId, 
			   int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			   int *bilan);

int vimanAbortRqstAndRcv (CLIENT_ID clientId, 
			     int *activity,
			     int *bilan);




extern STATUS vimanInitRqstSend (CLIENT_ID clientId, int *pRqstId,
					 VimanInitStr *(in_initStruct),
					 int replyTimeOut);

extern int vimanInitReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int vimanInitRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					VimanInitStr *(in_initStruct), 
					
					int *activity, int *bilan);

extern int vimanInitRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					VimanInitStr *(in_initStruct), 
					
					int *activity,
					int *bilan);




extern STATUS vimanSetTrackingUserModeRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 VimanBool *(in_flagUserTracking),
					 int replyTimeOut);

int vimanSetTrackingUserModeReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int vimanSetTrackingUserModeRqstAndRcv (CLIENT_ID clientId, 
				 VimanBool *(in_flagUserTracking), 
				 
				 int *bilan);




extern STATUS vimanActivateTrackingRqstSend (CLIENT_ID clientId, int *pRqstId,
					 
					 int replyTimeOut);

extern int vimanActivateTrackingReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int vimanActivateTrackingRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					 
					
					int *activity, int *bilan);

extern int vimanActivateTrackingRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					 
					
					int *activity,
					int *bilan);




extern STATUS vimanSetCameraLeftPoseRqstSend (CLIENT_ID clientId, int *pRqstId,
					 VimanThetaMat *(in_CAMERA_LEFT_ROB_POSE),
					 int replyTimeOut);

extern int vimanSetCameraLeftPoseReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int vimanSetCameraLeftPoseRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					VimanThetaMat *(in_CAMERA_LEFT_ROB_POSE), 
					
					int *activity, int *bilan);

extern int vimanSetCameraLeftPoseRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					VimanThetaMat *(in_CAMERA_LEFT_ROB_POSE), 
					
					int *activity,
					int *bilan);




extern STATUS vimanGetCameraLeftPoseRqstSend (CLIENT_ID clientId, int *pRqstId,
					 
					 int replyTimeOut);

extern int vimanGetCameraLeftPoseReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      VimanThetaMat *(out_CAMERA_LEFT_ROB_POSE),
				      int *activity, int *bilan);


extern int vimanGetCameraLeftPoseRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					 
					VimanThetaMat *(out_CAMERA_LEFT_ROB_POSE),
					int *activity, int *bilan);

extern int vimanGetCameraLeftPoseRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					 
					VimanThetaMat *(out_CAMERA_LEFT_ROB_POSE),
					int *activity,
					int *bilan);




extern STATUS vimanGetCameraRightPoseRqstSend (CLIENT_ID clientId, int *pRqstId,
					 
					 int replyTimeOut);

extern int vimanGetCameraRightPoseReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      VimanThetaMat *(out_CAMERA_RIGHT_ROB_POSE),
				      int *activity, int *bilan);


extern int vimanGetCameraRightPoseRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					 
					VimanThetaMat *(out_CAMERA_RIGHT_ROB_POSE),
					int *activity, int *bilan);

extern int vimanGetCameraRightPoseRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					 
					VimanThetaMat *(out_CAMERA_RIGHT_ROB_POSE),
					int *activity,
					int *bilan);




extern STATUS vimanSetImageFrameCorrectionRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 VimanCorrectionVec *(in_corImageFrame),
					 int replyTimeOut);

int vimanSetImageFrameCorrectionReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int vimanSetImageFrameCorrectionRqstAndRcv (CLIENT_ID clientId, 
				 VimanCorrectionVec *(in_corImageFrame), 
				 
				 int *bilan);




extern STATUS vimanSetRobotFrameCorrectionRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 VimanCorrectionVec *(in_corOriginFrame),
					 int replyTimeOut);

int vimanSetRobotFrameCorrectionReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int vimanSetRobotFrameCorrectionRqstAndRcv (CLIENT_ID clientId, 
				 VimanCorrectionVec *(in_corOriginFrame), 
				 
				 int *bilan);




extern STATUS vimanStopRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int vimanStopReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int vimanStopRqstAndRcv (CLIENT_ID clientId, 
				  
				 
				 int *bilan);




extern STATUS vimanGetObjectPoseRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 VimanString *(in_objectName),
					 int replyTimeOut);

int vimanGetObjectPoseReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       VimanThetaMat *(out_thetaMat),
			       int *bilan);

int vimanGetObjectPoseRqstAndRcv (CLIENT_ID clientId, 
				 VimanString *(in_objectName), 
				 VimanThetaMat *(out_thetaMat),
				 int *bilan);




extern STATUS vimanSetExportObjectNameRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 VimanString *(in_objectNameToExport),
					 int replyTimeOut);

int vimanSetExportObjectNameReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int vimanSetExportObjectNameRqstAndRcv (CLIENT_ID clientId, 
				 VimanString *(in_objectNameToExport), 
				 
				 int *bilan);




extern STATUS vimanGetFullObjectPoseRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 VimanString *(in_objectName),
					 int replyTimeOut);

int vimanGetFullObjectPoseReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       VimanObjectPublic *(out_objectPublic),
			       int *bilan);

int vimanGetFullObjectPoseRqstAndRcv (CLIENT_ID clientId, 
				 VimanString *(in_objectName), 
				 VimanObjectPublic *(out_objectPublic),
				 int *bilan);




extern STATUS vimanSetThresholdARToolKitRqstSend (CLIENT_ID clientId, int *pRqstId,
					 int *(in_thresholdARToolKit),
					 int replyTimeOut);

extern int vimanSetThresholdARToolKitReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int vimanSetThresholdARToolKitRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					int *(in_thresholdARToolKit), 
					
					int *activity, int *bilan);

extern int vimanSetThresholdARToolKitRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					int *(in_thresholdARToolKit), 
					
					int *activity,
					int *bilan);


#ifdef __cplusplus
}
#endif
#endif /* viman_MSG_LIB_H */

